using System;
using LowLevel;

//	TSIPSerialP
//	Interface to access NETCom Serial Port control
//	(c) Jordi Bartolome 2004
//	www.tolaemon.com

namespace TSIPSerialP
{
	/// <summary>
	/// SerialPort Class by J.Bartolome (Tolaemon) 30-3-04 Barcelona
	/// This Class is used as suport to NetComm control from Richard Grier which allows 
	/// you access  serial port the same way it was able with MSComm on Visual Studio 6. 
	/// This control is freeware and you should install it and register it in your computer 
	/// before using this code.
	/// </summary>
	

	public class SerialPort{				
		public ConversionFunctions Converter = new ConversionFunctions();
		public short port=1;//COM port number
		public int speed=9600;
		public uint parity=1;//0=None,1=Odd,2=Even
		public uint n_bits=8;//number of bits of data
		public uint stop_bits=1;//number of stop bits
		public string input_buffer;
		public string output_buffer;
		
		public SerialPort(){		
		
			//
			// Constructor
			//
		}
		
		//This function returns a string with Serial Port settings prepared to be
		//sent to the NETComm control. As you can see this function has not anything
		//special, in fact it's not too necessary, it could be done manually each time
		//you call NETComm set_settins method.
		public string init_serial(string _port, string _speed,string _parity,string _n_bits, string _stop_bits){	
			string aux_string;			
		
			// translation of parameters recived from combo boxes
			// COM port is taken form _port wich is the comboboxN text
			switch (_port){
				case "COM1":
					port=1;
					break;
				case "COM2":
					port=2;
					break;
				case "COM3":
					port=3;
					break;
				case "COM4":
					port=4;
					break;				
				default:
					port=1;
					break;
			};/*switch*/
			
			// speed is taken form _speed wich is the comboboxN text
			switch (_speed){
				case "2400":
					speed=2400;
					break;
				case "4800":
					speed=4800;
					break;
				case "9600":
					speed=9600;
					break;
				case "19200":
					speed=19200;
					break;				
				case "38400":
					speed=38400;
					break;				
				case "57600":
					speed=57600;
					break;				
				default:
					speed=9600;
					break;
			};/*switch*/

			// parity is taken form _parity wich is the comboboxN text
			switch (_parity){
				case "None":
					parity=0;
					break;
				case "Odd":
					parity=1;
					break;
				case "Even":
					parity=2;
					break;
				default:
					parity=1;
					break;
			};/*switch*/

			//numberdata bits is taken form _n_bits wich is the comboboxN text
			switch (_n_bits){
				case "7":
					n_bits=7;				
					break;
				case "8":
					n_bits=8;
					break;
				case "9":
					n_bits=9;
					break;
			};/*switch*/

			//Stop_Bits is taken from comoboBox5
			switch (_stop_bits){
				case "0":
					stop_bits=0;				
					break;
				case "1":
					stop_bits=1;				
					break;
				case "2":
					stop_bits=2;				
					break;
			};/*switch*/

			//after translating configuration parameters from combo boxes they are 
			//all set into a string wich will be usefull to send config to axNETcomm
			aux_string= _speed.ToString();//speed is add to aux_string			
			switch (parity){
				case 0://None
					aux_string= aux_string + ",N";//,N is add to aux_string
					break;
				case 1://Odd
					aux_string= aux_string + ",O";//,O is add to aux_string
					break;
				case 2://Even
					aux_string= aux_string + ",E";//,E is add to aux_string
					break;
				default:
					aux_string= aux_string + ",N";//,N is add to aux_string
					break;
			};//switch
			aux_string=aux_string + "," + _n_bits.ToString();//number of data bits is add
			aux_string=aux_string + "," + _stop_bits.ToString();//number of stop_bits is add
			return aux_string;//string with settins is returned		
		
		}//init_serial

 
		//This method recives a string with a variable number of 2 digit hexadecimal values
		//and converts it into a bytes stream prepared to be sent through serial port.
		//Ex: [FF][01][A0][BB][02]  => 255 1 160 187 02
		public void write_output_buffer(string to_send){
			char [] aux_char_array = new char [2];
			bool leave_while=false;
			int i=0;

			output_buffer="";//first output_buffer is flushed			
			while (!leave_while){//while leave_while is true we stay in the loop
				i++;
				if (i>=to_send.Length){//we have arrived to the end of the string: we can leave the loop
					leave_while=true;
				}else{
					if (to_send[i]==']'){
						//when read character is ']' it means that there is a hexadecimal byte to be read (two hexadecimal digits)
						if (i>=2){
							//if is >=2 reading positions i-2 and i-1 wont cause error
							aux_char_array[0]=to_send[i-2];
							aux_char_array[1]=to_send[i-1];						
							output_buffer=output_buffer + (Converter.ByteHexToASCII(aux_char_array));
						}else{
							//if is <2 reading positions i-2 and i-1 will cause error, so we leave the loop
							leave_while=true;					
						};//if
					};//if
				};//if
			};//while					
			
		}//write_output_buffer
	}
}
